package net.sf.sido;

import net.sf.jstring.JStrings;
import net.sf.jstring.Localizable;
import org.apache.commons.lang.StringUtils;

import java.util.Locale;

public abstract class DOException extends RuntimeException implements Localizable {

	private final DOKey key;
	private final Object[] parameters;

	public DOException(DOKey key, Object... parameters) {
		super(key.toString() + " " + StringUtils.join(parameters, ","));
		this.key = key;
		this.parameters = parameters;
	}

	public DOException(DOKey key, Throwable th, Object... parameters) {
		this(key, parameters);
		initCause(th);
	}

	public DOKey getKey() {
		return this.key;
	}

	public Object[] getParameters() {
		return this.parameters;
	}

    @Override
    public String getLocalizedMessage() {
        return getLocalizedMessage(Locale.getDefault());
    }

    @Override
    public String getLocalizedMessage(Locale locale) {
        String skey = "sido." + key;
        return JStrings.get(locale, skey, parameters);
    }
}
