package net.sf.sido;

/**
 * Defines a difference on a property.
 */
public class Difference {

    private final String property;
    private final Object value1;
    private final Object value2;

    /**
     * Creates a new difference
     * @param property Property path
     * @param value1 Value 1
     * @param value2 Value 2
     */
    public Difference(String property, Object value1, Object value2) {
        this.property = property;
        this.value1 = value1;
        this.value2 = value2;
    }

    /**
     * Gets the property path. It can include [i] for array elements.
     * @return Property path
     */
    public String getProperty() {
        return property;
    }

    /**
     * Value for the first object
     * @return Value
     */
    public Object getValue1() {
        return value1;
    }

    /**
     * Value for the second object
     * @return Value
     */
    public Object getValue2() {
        return value2;
    }

    public Difference prefix (String prefix) {
        String newKey = prefix + property;
        return new Difference (newKey, value1, value2);
    }

    @Override
    public String toString() {
        return String.format("%s[%s/%s]", property, value1, value2);
    }

}
