package net.sf.sido.factory.json;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

import net.sf.sido.DataFactory;
import net.sf.sido.DataSchema;

public class JSONReaderContext {

	private final DataSchema schema;

	private final Map<String, JSONTypeDefinition> definitions = new LinkedHashMap<String, JSONTypeDefinition>();

	public JSONReaderContext(DataSchema schema) {
		this.schema = schema;
	}

	public void addDefinition(JSONTypeDefinition definition) {
		String name = definition.getName();
		if (this.definitions.containsKey(name)) {
			throw new IllegalStateException("Type " + name + " is already defined.");
		} else {
			this.definitions.put(name, definition);
		}
	}

	public JSONTypeDefinition getDefinition(String name) {
		return this.definitions.get(name);
	}

	public Collection<JSONTypeDefinition> getDefinitions() {
		return this.definitions.values();
	}

	public DataFactory getFactory() {
		return this.schema.getFactory();
	}

	public DataSchema getSchema() {
		return this.schema;
	}

}
