package net.sf.sido.factory.sido.antlr;

import java.util.ArrayList;
import java.util.List;

import net.sf.sido.DataSchema;
import net.sf.sido.DataType;

import org.apache.commons.lang.StringUtils;

public abstract class AbstractPropertyContainer implements IPropertyContainer {

	protected final DataSchema schema;

	private Property property;
	private final List<Property> properties = new ArrayList<Property>();

	public AbstractPropertyContainer(DataSchema schema) {
		this.schema = schema;
	}

	@Override
	public void pushNewProperty() {
		property = new Property();
	}

	@Override
	public void pushPropertyNullable(boolean not) {
		property.setNullable(not);
	}

	@Override
	public void pushAnonymousProperty() {
		property.setAnonymous(true);
	}

	@Override
	public void pushProperty(String name) {
		property.setName(name);
		properties.add(property);
	}

	@Override
	public void pushPropertyReverse() {
		property.setReverse();
	}

	@Override
	public void pushPropertyArray() {
		property.setArray(true);
	}

	@Override
	public void pushPropertyType(String typeName) {
		property.setTypeName(typeName);
	}

	protected DataType getType(String typeName) {
		if (StringUtils.isBlank(typeName)) {
			return null;
		} else if (typeName.contains(DataSchema.QUALIFIER_SEPARATOR)) {
			return schema.getFactory().getType(typeName);
		} else {
			return schema.getType(typeName);
		}
	}

	@Override
	public void resolve() {
		for(Property property : properties) {
			resolveProperty(property);
		}
	}

	@Override
	public void pushConstraint(String id) {
		if (property == null) {
			throw new IllegalStateException("No property is available");
		} else {
			property.pushConstraint(id);
		}
	}

	@Override
	public void pushConstraintValue(String value) {
		if (property == null) {
			throw new IllegalStateException("No property is available");
		} else {
			property.pushConstraintValue(value);
		}
	}

	@Override
	public void pushConstraintParam(String name, String value) {
		if (property == null) {
			throw new IllegalStateException("No property is available");
		} else {
			property.pushConstraintParam(name, value);
		}
	}

	protected abstract void resolveProperty(Property property);

}
