package net.sf.sido.factory.sido.antlr;

import net.sf.sido.DOTypeNotFoundException;
import net.sf.sido.DataFactory;
import net.sf.sido.DataObject;
import net.sf.sido.DataSchema;
import net.sf.sido.DataType;
import net.sf.sido.array.Array;

import org.apache.commons.lang.StringUtils;

import com.google.common.base.Predicate;

public class ExtensionPropertyContainer extends AbstractPropertyContainer {

	private final DataType extendedType;

	public ExtensionPropertyContainer(DataSchema schema, String extendedTypeName) {
		super(schema);
		extendedType = getType(extendedTypeName);
		if (extendedType == null) {
			throw new DOTypeNotFoundException(extendedTypeName);
		}
	}

	@Override
	public void pushAnonymousProperty() {
		throw new IllegalStateException("Anonymous properties are not allowed for extensions.");
	}

	@Override
	protected void resolveProperty(Property property) {
		DataFactory factory = schema.getFactory();
		// Property declaration
		String name = property.getName();
		String typeName = property.getTypeName();
		boolean array = property.isArray();
		boolean nullable = property.isNullable();
		boolean reverse = property.isReverse();
		// Default type
		if (StringUtils.isBlank(typeName)) {
			typeName = "string";
		}
		// Anonymous properties are not allowed for extensions
		if (property.isAnonymous()) {
			throw new IllegalStateException("Anonymous properties are not allowed for extensions.");
		}
		// Constraint
		Predicate<?> constraint = factory.getConstraintFactory().createConstraint(property.getConstraintDefinitions());
		// Property type
		DataType doType = getType(typeName);
		boolean simpleType = (doType == null);
		if (simpleType) {
			Class<?> typeClass = factory.getSimpleType(typeName);
			// Array
			if (array) {
				// Simple array
				extendSimpleArray(name, nullable, typeClass, constraint);
			}
			// Single
			else {
				// Simple
				extendSimple(name, nullable, typeClass, constraint);
			}
		}
		// DO type
		else {
			// Array
			if (array) {
				@SuppressWarnings("unchecked")
				Predicate<Array<DataObject>> thisConstraint = (Predicate<Array<DataObject>>) constraint;
				extendedType.extendArray(name, doType, nullable, thisConstraint);
			}
			// Single
			else {
				@SuppressWarnings("unchecked")
				Predicate<DataObject> thisConstraint = (Predicate<DataObject>) constraint;
				extendedType.extend(name, doType, nullable, reverse, thisConstraint);
			}
		}
	}

	protected <T> void extendSimpleArray(String name, boolean nullable, Class<T> typeClass, Predicate<?> constraint) {
		@SuppressWarnings("unchecked")
		Predicate<Array<T>> thisConstraint = (Predicate<Array<T>>) constraint;
		extendedType.extendArray(name, typeClass, nullable, thisConstraint);
	}

	protected <T> void extendSimple(String name, boolean nullable, Class<T> typeClass, Predicate<?> constraint) {
		@SuppressWarnings("unchecked")
		Predicate<T> thisConstraint = (Predicate<T>) constraint;
		extendedType.extend(name, typeClass, nullable, thisConstraint);
	}

}
